@@ -36,6 +36,14 @@
 #include <string.h>
 #include <libxfce4util/libxfce4util.h>
 
+#ifdef HAVE_LIBDRM
+#include <stdint.h>
+#include <fcntl.h>
+#include <errno.h>
+#include <drm.h>
+#include <stropts.h>
+#endif /* HAVE_LIBDRM */
+
 #include "display.h"
 #include "screen.h"
 #include "client.h"
@@ -86,7 +94,12 @@
 #define WIN_IS_REDIRECTED(cw)           (cw->redirected)
 
 /* Set TIMEOUT_REPAINT to 0 to disable timeout repaint */
-#define TIMEOUT_REPAINT       10 /* msec.) */
+#define TIMEOUT_REPAINT       10 /* msec */
+#define TIMEOUT_REPAINT_MIN    1
+#define TIMEOUT_REPAINT_MAX   20
+#define TIMEOUT_DRI           10 /* seconds */
+
+#define DRM_CARD0             "/dev/dri/card0"
 
 typedef struct _CWindow CWindow;
 struct _CWindow
@@ -1247,6 +1260,113 @@ paint_win (CWindow *cw, XserverRegion region, gboolean solid_part)
     }
 }
 
+#if HAVE_LIBDRM
+#if TIMEOUT_REPAINT
+
+static void
+open_dri (ScreenInfo *screen_info)
+{
+    screen_info->dri_fd = open (DRM_CARD0, O_RDWR);
+    if (screen_info->dri_fd == -1)
+    {
+        g_warning ("Error opening %s: %s", DRM_CARD0, g_strerror (errno));
+    }
+}
+
+static void
+close_dri (ScreenInfo *screen_info)
+{
+    if (screen_info->dri_fd != -1)
+    {
+        close (screen_info->dri_fd);
+        screen_info->dri_fd = -1;
+    }
+}
+
+static gboolean
+dri_enabled (ScreenInfo *screen_info)
+{
+    return (screen_info->dri_fd != -1 && screen_info->params->sync_to_vblank);
+}
+
+static void
+wait_vblank (ScreenInfo *screen_info)
+{
+    int retval;
+    drm_wait_vblank_t vblank;
+
+    if (screen_info->dri_time > g_get_monotonic_time())
+    {
+        return;
+    }
+
+    vblank.request.sequence = 1;
+    vblank.request.type = _DRM_VBLANK_RELATIVE;
+    if (screen_info->dri_secondary)
+    {
+        vblank.request.type |= _DRM_VBLANK_SECONDARY;
+    }
+
+    do
+    {
+       retval = ioctl (screen_info->dri_fd, DRM_IOCTL_WAIT_VBLANK, &vblank);
+       vblank.request.type &= ~_DRM_VBLANK_RELATIVE;
+    }
+    while (retval == -1 && errno == EINTR);
+
+    screen_info->vblank_time = g_get_monotonic_time ();
+
+    if (retval == -1)
+    {
+        if (screen_info->dri_success)
+        {
+            screen_info->dri_success = FALSE;
+            g_warning ("Error waiting on vblank with DRI: %s", g_strerror (errno));
+        }
+
+        /* if getting the vblank fails, try to get it from the other output */
+        screen_info->dri_secondary = !screen_info->dri_secondary;
+
+        /* the output that we tried to get the vblank from might be disabled,
+           if that's the case, the device needs to be reopened, or it will continue to fail */
+        close_dri (screen_info);
+        open_dri (screen_info);
+
+        /* retry in 10 seconds */
+        screen_info->dri_time = g_get_monotonic_time() + TIMEOUT_DRI * 1000000;
+    }
+    else if (!screen_info->dri_success)
+    {
+        g_message ("Using vertical blank of %s DRI output",
+                   screen_info->dri_secondary ? "secondary" : "primary");
+
+        screen_info->dri_success = TRUE;
+    }
+}
+
+#ifdef HAVE_RANDR
+static void
+get_refresh_rate (ScreenInfo* screen_info)
+{
+    gint refresh_rate;
+    XRRScreenConfiguration* randr_info;
+
+    randr_info = XRRGetScreenInfo (screen_info->display_info->dpy, screen_info->xroot);
+    refresh_rate = XRRConfigCurrentRate (randr_info);
+    XRRFreeScreenConfigInfo (randr_info);
+
+    if (refresh_rate != screen_info->refresh_rate)
+    {
+        DBG ("Detected refreshrate: %i hertz", refresh_rate);
+        screen_info->refresh_rate = refresh_rate;
+    }
+}
+#endif /* HAVE_RANDR */
+
+#endif /* TIMEOUT_REPAINT */
+
+#endif /* HAVE_LIBDRM */
+
 static void
 paint_all (ScreenInfo *screen_info, XserverRegion region)
 {
@@ -1258,6 +1378,12 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
     gint screen_height;
     CWindow *cw;
 
+#ifdef HAVE_LIBDRM
+#if TIMEOUT_REPAINT
+    gboolean use_dri;
+#endif /* TIMEOUT_REPAINT */
+#endif /* HAVE_LIBDRM */
+
     TRACE ("entering paint_all");
     g_return_if_fail (screen_info);
 
@@ -1406,8 +1532,34 @@ paint_all (ScreenInfo *screen_info, XserverRegion region)
     TRACE ("Copying data back to screen");
     /* Set clipping back to the given region */
     XFixesSetPictureClipRegion (dpy, screen_info->rootBuffer, 0, 0, region);
+
+#ifdef HAVE_LIBDRM
+#if TIMEOUT_REPAINT
+    use_dri = dri_enabled (screen_info);
+    
+    if (use_dri)
+    {
+        /* sync all previous rendering commands, tell xlib to render the pixmap
+         * onto the root window, wait for the vblank, then flush, this minimizes
+         * tearing*/
+        XFlush (dpy);
+    }
+#endif /* TIMEOUT_REPAINT */
+#endif /* HAVE_LIBDRM */
+
     XRenderComposite (dpy, PictOpSrc, screen_info->rootBuffer, None, screen_info->rootPicture,
                       0, 0, 0, 0, 0, 0, screen_width, screen_height);
+
+#ifdef HAVE_LIBDRM
+#if TIMEOUT_REPAINT
+    if (use_dri)
+    {
+        wait_vblank (screen_info);
+        XFlush (dpy);
+    }
+#endif /* TIMEOUT_REPAINT */
+#endif /* HAVE_LIBDRM */
+
     XFixesDestroyRegion (dpy, paint_region);
 }
 
@@ -1467,13 +1619,57 @@ static void
 add_repair (ScreenInfo *screen_info)
 {
 #if TIMEOUT_REPAINT
+#ifdef HAVE_LIBDRM
+    gint64 interval;
+#endif /* HAVE_LIBDRM */
+
     if (screen_info->compositor_timeout_id != 0)
     {
         return;
     }
+
+#ifdef HAVE_LIBDRM
+    if (dri_enabled (screen_info))
+    {
+        /* schedule the next render to be half a refresh period after the last vertical blank,
+           but at least 1 ms in the future so that all queued events can be processed,
+           and to reduce latency if we didn't render for a while */
+#ifdef HAVE_RANDR
+        if (screen_info->refresh_rate > 0)
+        {
+            interval = (screen_info->vblank_time + 500000 / screen_info->refresh_rate -
+                        g_get_monotonic_time ()) / 1000;
+        }
+        else
+#endif /* HAVE_RANDR */
+        {
+            interval = TIMEOUT_REPAINT - ((g_get_monotonic_time () - screen_info->vblank_time) / 1000);
+        }
+
+        if (interval > TIMEOUT_REPAINT_MAX)
+        {
+            interval = TIMEOUT_REPAINT_MAX;
+        }
+        else if (interval < TIMEOUT_REPAINT_MIN)
+        {
+            interval = TIMEOUT_REPAINT_MIN;
+        }
+    }
+    else
+    {
+        interval = TIMEOUT_REPAINT;
+    }
+#endif /* HAVE_LIBDRM */
+
     screen_info->compositor_timeout_id =
+#ifdef HAVE_LIBDRM
+        g_timeout_add (interval,
+                       compositor_timeout_cb, screen_info);
+#else
         g_timeout_add (TIMEOUT_REPAINT,
                        compositor_timeout_cb, screen_info);
+#endif /*HAVE_LIBDRM */
+
 #endif /* TIMEOUT_REPAINT */
 }
 
@@ -2543,6 +2739,26 @@ compositorHandleShapeNotify (DisplayInfo *display_info, XShapeEvent *ev)
     }
 }
 
+#ifdef HAVE_LIBDRM
+#ifdef HAVE_RANDR
+static void
+compositorHandleRandrNotify (DisplayInfo *display_info, XRRScreenChangeNotifyEvent *ev)
+{
+    ScreenInfo *screen_info;
+
+    g_return_if_fail (display_info != NULL);
+    g_return_if_fail (ev != NULL);
+    TRACE ("entering compositorHandleRandrNotify for 0x%lx", ev->window);
+
+    screen_info = myDisplayGetScreenFromRoot (display_info, ev->window);
+    if (screen_info)
+        get_refresh_rate (screen_info);
+
+    XRRUpdateConfiguration ((XEvent *) ev);
+}
+#endif /* HAVE_RANDR */
+#endif /* HAVE_LIBDRM */
+
 static void
 compositorSetCMSelection (ScreenInfo *screen_info, Window w)
 {
@@ -2753,6 +2969,15 @@ compositorHandleEvent (DisplayInfo *display_info, XEvent *ev)
     {
         compositorHandleShapeNotify (display_info, (XShapeEvent *) ev);
     }
+#ifdef HAVE_LIBDRM
+#ifdef HAVE_RANDR
+    else if (ev->type == (display_info->xrandr_event_base + RRScreenChangeNotify))
+    {
+        compositorHandleRandrNotify (display_info, (XRRScreenChangeNotifyEvent *) ev);
+    }
+#endif /* HAVE_RANDR */
+#endif /* HAVE_LIBDRM */
+
 #if TIMEOUT_REPAINT == 0
     repair_display (display_info);
 #endif /* TIMEOUT_REPAINT */
@@ -2971,6 +3196,22 @@ compositorManageScreen (ScreenInfo *screen_info)
     compositorSetCMSelection (screen_info, screen_info->xfwm4_win);
     TRACE ("Manual compositing enabled");
 
+#ifdef HAVE_LIBDRM
+    open_dri (screen_info);
+    screen_info->dri_success = TRUE;
+    screen_info->dri_secondary = FALSE;
+    screen_info->dri_time = 0;
+    screen_info->vblank_time = 0;
+
+#ifdef HAVE_RANDR
+    if (display_info->have_xrandr)
+    {
+        get_refresh_rate(screen_info);
+        XRRSelectInput(display_info->dpy, screen_info->xroot, RRScreenChangeNotifyMask);
+    }
+#endif /* HAVE_RANDR */
+#endif /* HAVE_LIBDRM */
+
     return TRUE;
 #else
     return FALSE;
@@ -3063,6 +3304,18 @@ compositorUnmanageScreen (ScreenInfo *screen_info)
                                     display_info->composite_mode);
 
     compositorSetCMSelection (screen_info, None);
+
+#ifdef HAVE_LIBDRM
+    close_dri (screen_info);
+
+#ifdef HAVE_RANDR
+    if (display_info->have_xrandr)
+    {
+        XRRSelectInput (display_info->dpy, screen_info->xroot, 0);
+    }
+#endif /* HAVE_RANDR */
+#endif /* HAVE_LIBDRM */
+
 #endif /* HAVE_COMPOSITOR */
 }
 
