@@ -181,6 +181,19 @@ struct _ScreenInfo
     gboolean damages_pending;
 
     guint compositor_timeout_id;
+
+#ifdef HAVE_LIBDRM
+    gint dri_fd;
+    gboolean dri_secondary;
+    gboolean dri_success;
+    gint64 dri_time;
+    gint64 vblank_time;
+
+#ifdef HAVE_RANDR
+    gint refresh_rate;
+#endif /* HAVE_RANDR */
+#endif /* HAVE_LIBDRM */
+
 #endif /* HAVE_COMPOSITOR */
 };
 
@@ -212,6 +225,7 @@ void                     myScreenGrabKeys                       (ScreenInfo *);
 void                     myScreenUngrabKeys                     (ScreenInfo *);
 int                      myScreenGetKeyPressed                  (ScreenInfo *,
                                                                  XKeyEvent *);
+int                      myScreenGetModifierPressed             (ScreenInfo *);
 Client                  *myScreenGetClientFromWindow            (ScreenInfo *,
                                                                  Window,
                                                                  unsigned short);
