@@ -720,7 +720,9 @@ loadSettings (ScreenInfo *screen_info)
         {"snap_to_border", NULL, G_TYPE_BOOLEAN, TRUE},
         {"snap_to_windows", NULL, G_TYPE_BOOLEAN, TRUE},
         {"snap_width", NULL, G_TYPE_INT, TRUE},
+        {"sync_to_vblank", NULL, G_TYPE_BOOLEAN, TRUE},
         {"theme", NULL, G_TYPE_STRING, TRUE},
+        {"tile_on_move", NULL, G_TYPE_BOOLEAN, TRUE},
         {"title_alignment", NULL, G_TYPE_STRING, TRUE},
         {"title_font", NULL, G_TYPE_STRING, FALSE},
         {"title_horizontal_offset", NULL, G_TYPE_INT, TRUE},
@@ -822,6 +824,10 @@ loadSettings (ScreenInfo *screen_info)
         getBoolValue ("snap_resist", rc);
     screen_info->params->snap_width =
         getIntValue ("snap_width", rc);
+    screen_info->params->sync_to_vblank =
+        getBoolValue ("sync_to_vblank", rc);
+    screen_info->params->tile_on_move =
+        getBoolValue ("tile_on_move", rc);
     screen_info->params->toggle_workspaces =
         getBoolValue ("toggle_workspaces", rc);
     screen_info->params->unredirect_overlays =
@@ -1322,6 +1328,14 @@ cb_xfwm4_channel_property_changed(XfconfChannel *channel, const gchar *property_
                 {
                     screen_info->params->snap_resist = g_value_get_boolean (value);
                 }
+                else if (!strcmp (name, "tile_on_move"))
+                {
+                    screen_info->params->tile_on_move = g_value_get_boolean (value);
+                }
+                else if (!strcmp (name, "sync_to_vblank"))
+                {
+                    screen_info->params->sync_to_vblank = g_value_get_boolean (value);
+                }
                 else if (!strcmp (name, "toggle_workspaces"))
                 {
                     screen_info->params->toggle_workspaces = g_value_get_boolean (value);
